#! /usr/bin/python

import sys
import tfm  # transform library

if len(sys.argv) <= 1:
    print >>sys.stderr, """Usage:  %s  infile.wf  > outfile.matrices
Reads a virdir-style .wf animation-path file,
writes to stdout a series of 4x4 camera-to-world matrices in OpenGL style
(4 lines per matrix, with a blank line after each one).

Input format:
Each line of a .wf animation-path file contains six or seven ASCII numbers, representing

   px py pz rx ry rz [fovy]
Meaning:
   px py pz -- position of the camera in "world" coordinates
   rx ry rz -- Euler angles, in degrees, intended to be multiplied in "yxz" order:
   fovy (optional) -- camera's field-of-view in Y direction

Interpretation of the p's and r's:

      p_camera * RotZ[rz] * RotX[rx] * RotY[ry] * Translation[px,py,pz] = p_world

where p_camera is a point in camera coordinates
and p_world is the same point expressed in world coordinates.

(Writing the transformations this way avoids ambiguity about things like "multiplication on the right".)


Output format: each line of input is converted to a 4x4 camera-to-world matrix,
in 4 lines of 4 numbers plus a blank line.   Please edit the script if another form would be handier.

Camera-to-world matrices are intended to be multiplied by camera-space points as in:
   p_world = p_camera * Tc2w

like
  txx txy txz 0
  tyx tyy tyz 0
  tzx tzy tzz 0
  px  py  pz  1

where as by opengl convention,
   +X in camera space is rightward,
   +Y is upward,
   +Z is *backward* (so -Z is camera forward)
Matrices are orthonormal (unit scale in each direction).
Thus the first 3 entries of the matrix's first row (txx txy txz)
    form a unit world-space vector toward the camera's +X direction,
second row is a unit world-space vector toward camera +Y, etc.
The fourth row gives the camera's position in world coordinates.
""" % sys.argv[0]

    sys.exit(1)

wffile = open(sys.argv[1])

tfmt = 4 * "%15.9g %15.9g %15.9g %15.9g\n" 

for rawline in wffile.readlines():
    line = rawline.split('#')[0]   # discard anything past a '#' comment
    vals = map( float, line.split() )
    if len(vals) >= 6:
	px,py,pz, rx,ry,rz = vals[:6]
	Tc2w = tfm.vd2tfm( px,py,pz, rx,ry,rz )
	print "# %s" % rawline[:-1]  # Include input line as a '#' comment.  Omit trailing newline.
	print tfmt % tuple(Tc2w.ravel())
